#include "Generateur.h"

using namespace std;

/**
* Constructeur d'un objet Generateur
* @param prefixe_i :		prfixe de sauvegarde des images gnres
* @param nbCliche_i : 		nombre de clichs  gnrer
* @param bitmapTailleX_i : 	taille horizontale des images
* @param bitmapTailleY_i : 	taille verticale des images
* @param nbNoeud_i : 		nombre de noeuds de la grille
*/

Generateur::Generateur(const string prefixe_i, const unsigned int nbCliche_i,
			const unsigned int bitmapTailleX_i, const unsigned int bitmapTailleY_i,
			const unsigned int nbNoeud_i) :
		prefixe(string(prefixe_i)), nbCliche(nbCliche_i),
		bitmapTailleX(bitmapTailleX_i), bitmapTailleY(bitmapTailleY_i),
		nbNoeudX(nbNoeud_i), nbNoeudY(nbNoeud_i)
{
	cliche = new Cliche(bitmapTailleX, bitmapTailleY);
}

/**
* Destructeur d'un objet Generateur
*/

Generateur::~Generateur()
{
	if(cliche)
		delete cliche;
}

/**
* Gnre un fichier avec la position initiale de chaque noeud de la grille
* @param fichierNom : nom du fichier
* @return bool : succs
*/

bool Generateur::sauvegarderPos(const string& fichierNom)
{
	ofstream fichierPos;
	unsigned int i;
	unsigned int x, y;
	
	fichierPos.open(fichierNom.c_str(), ios::trunc);
	
	fichierPos << nbNoeudX << "," << nbNoeudY << endl;
	fichierPos << "#" << endl;
	
	for(i=0; i<noeudTab.size(); i++)
	{
		x = noeudTab[i].getX();
		y = noeudTab[i].getY();
		
		fichierPos << x << "," << y << endl;
	}
			
	fichierPos.close();
	
	return true;
}

/**
* Genere une squence d'images
* @return Bool : succs
*/

bool Generateur::generer()
{
	unsigned int i, j;
	int x0, y0;
	double a, b;
	double w1, w2;
	
	// Cre un ensemble de noeuds

	for(i=0; i<nbNoeudY; i++)
	{
		for(j=0; j<nbNoeudX; j++)
		{
			x0 = (int)((double)(j+1) / (nbNoeudX+2) * bitmapTailleX);
			y0 = (int)((double)(i+1) / (nbNoeudY+2) * bitmapTailleY);

			a = ((rand()%1000)/2000.0f + 0.3f ) * bitmapTailleX / (2 * (nbNoeudX + 1));
			b = ((rand()%1000)/2000.0f + 0.3f ) * bitmapTailleY / (2 * (nbNoeudY + 1));
			w1 = (rand()%1000)/1000.0f + 0.5f;
			w2 = (rand()%1000)/1000.0f + 0.5f;

			noeudTab.push_back(Noeud(a, b, w1, w2, x0, y0));
		}
	}

	// Gnre n cliches

	for(i=0; i<nbCliche; i++)
	{
		ostringstream fichierNom;
		
		for(j=0; j<noeudTab.size(); j++)
			noeudTab[j].generer(i);

		if(i == 0)
		{
			fichierNom << prefixe << setfill('0') << setw(4) << i << ".pos";
			
			sauvegarderPos(fichierNom.str());
		}
		else
		{
			cliche->generer(noeudTab);
	
			fichierNom << prefixe << setfill('0') << setw(4) << i << ".ppm";
			cliche->sauvegarder(fichierNom.str());
		}
	}
	
	return true;
}

