#include "Image.h"

using namespace std;

/**
* Constructeur d'un objet Image
*/

Image::Image()
{
	tailleX = 0;
	tailleY = 0;
	BPP = 0;
	
	bitmapTaille = 0;
	bitmap = NULL;

        fichierTmpTailleMax = 10000;
        fichierTmp = (char *)malloc(fichierTmpTailleMax);
}

/**
* Constructeur d'un objet Image
* @param tailleX_i : largeur de l'image
* @param tailleY_i : hauteur de l'image
* @param BPP_i : nombre de bits par pixel
*/

Image::Image(const unsigned int tailleX_i, const unsigned int tailleY_i, const unsigned int BPP_i)
{
	tailleX = tailleX_i;
	tailleY = tailleY_i;
	BPP = BPP_i;
	
	bitmapTaille = sizeof(unsigned char) * tailleX * tailleY * BPP / 8;
	bitmap = (unsigned char *)malloc(bitmapTaille);
	
        fichierTmpTailleMax = 10000;
        fichierTmp = (char *)malloc(fichierTmpTailleMax);
}

/**
* Destructeur d'un objet Image
*/

Image::~Image()
{
	if(bitmap)
		free(bitmap);
	
	if(fichierTmp)
		free(fichierTmp);
}

/**
* Attribue une couleur uniforme  l'image
* @param couleur : couleur attribue
* @return bool : succs
*/

bool Image::remplir(unsigned char couleur)
{
	memset(bitmap, couleur, bitmapTaille);

	return true;
}

/**
* Sauvegarde l'image au format brut 
* @param fichierNom : nom du fichier
* @return bool : succs
*/

bool Image::sauvegarderRAW(const string& fichierNom)
{
	if((fichier = fopen(fichierNom.c_str(), "wb")) == NULL)
	{
		cerr << "Erreur de cration du fichier " << fichierNom << endl;

		return 0;
	}

	fwrite(bitmap, 1, tailleX * tailleY, fichier);

	fclose(fichier);

	return true;
}

/*void Image::bufferLit()
{
	do
	{
		fgets(fichierTmp, fichierTmpTailleMax, fichier);
	}
	while(fichierTmp[0] == '#');
}

bool Image::chargerPGM(const string& fichierNom)
{
	int i, j;

	// Ouverture fichier

	if(!(fichier = fopen(fichierNom.c_str(), "r")))
	{
		cerr << "Image : Impossible d'ouvrir le fichier " << fichierNom << endl;

		return false;
	}

	// Lecture entete

	bufferLit();

	if(strncmp(fichierTmp, "P5", 2) == 0)
		BPP = 32;
	else
	if(strncmp(fichierTmp, "P6", 2) == 0)
		BPP = 8;
	else
	{
		cerr << "Image : Format non reconnu" << endl;

		return false;
	}

	// Largeur/hauteur

	bufferLit();
	sscanf(fichierTmp, "%d %d", &tailleX, &tailleY);

	// Allocation
	
	if(!bitmap)
	{
		if(BPP == 8)
			bitmap = (unsigned char *)malloc(tailleX * tailleY);
		else
			bitmap = (unsigned char *)malloc(tailleX * tailleY * 4);
	}

	// Taille palette (255).

	bufferLit();

	// Lecture image

	if(BPP == 8)
	{
		for(i=0; i<(int)tailleY; i++)
			for(j=0; j<(int)tailleX; j++)
				bitmap[i*tailleX+j] = (unsigned char)fgetc(fichier);
	}
	else
	{
		for(i=0; i<(int)tailleY; i++)
			for(j=0; j<(int)tailleX; j++)
			{
				bitmap[((i*tailleX+j) << 2)] = (unsigned char)fgetc(fichier);
				bitmap[((i*tailleX+j) << 2)+1] = (unsigned char)fgetc(fichier);
				bitmap[((i*tailleX+j) << 2)+2] = (unsigned char)fgetc(fichier);
				bitmap[((i*tailleX+j) << 2)+3] = 0;
			}
	}

	// Fini

	fclose(fichier);
	
	return true;
}

bool Image::sauvegarderPGM(const string& fichierNom)
{
	int i, j;

	// Cration fichier

	if (!(fichier = fopen(fichierNom.c_str(), "w")))
	{
		cerr << "Image : Impossible d'ouvrir le fichier " << fichierNom << " en criture" << endl;
		
		return false;
	}

	// criture entete

	if(BPP == 8)
		fprintf(fichier, "P5\n# CREATOR: M1INFO.\n");
	else
		fprintf(fichier, "P6\n# CREATOR: M1INFO.\n");

	fprintf(fichier, "%d %d\n255\n", tailleX, tailleY);

	// criture des donnees

	if(BPP == 8)
	{
		for(i=0; i<(int)tailleY; i++)
			for(j=0; j<(int)tailleX; j++)
				fprintf(fichier, "%c", bitmap[j + i*tailleX]);
	}
	else
	{
		for(i=0; i<(int)tailleY; i++)
			for(j=0; j<(int)tailleX; j++)
				fprintf(fichier, "%c%c%c",
					bitmap[((j + i*tailleX) << 2)],
					bitmap[((j + i*tailleX) << 2) + 1],
					bitmap[((j + i*tailleX) << 2) + 2]);
	}

	// Fini

	fclose(fichier);
	
	return true;
}
*/

/**
* Charge l'image au format PPM : noir et blanc, 1 bit par pixel
* @param fichierNom : nom du fichier
* @return bool : succs
*/

bool Image::chargerPPM(const string& fichierNom)
{
	unsigned int lu;
	char fichierTmp[255];

	// Ouverture fichier

	if(!(fichier = fopen(fichierNom.c_str(), "r")))
	{
		cerr << "Image : Impossible d'ouvrir le fichier " << fichierNom << endl;

		return false;
	}

	// Lecture entete

	fgets(fichierTmp, 255, fichier);

	if(strncmp(fichierTmp, "P4", 2) == 0)
		BPP = 1;
	else
	{
		cerr << "Image : Format non reconnu" << endl;

		return false;
	}

	// Largeur/hauteur

	fgets(fichierTmp, 255, fichier);
	fgets(fichierTmp, 255, fichier);
	sscanf(fichierTmp, "%d %d", &tailleX, &tailleY);

	// Allocation
	
	if(bitmap)
		free(bitmap);
	
	bitmapTaille = sizeof(unsigned char) * tailleX * tailleY * BPP / 8;
	bitmap = (unsigned char *)malloc(bitmapTaille);
	
	if(!bitmap)
	{
		cerr << "Image : Erreur d'allocation mmoire" << endl;

		return false;
	}

	// Lecture image

	lu = fread(bitmap, 1, bitmapTaille, fichier);

	if(lu != bitmapTaille)
	{
		cerr << "Image : Erreur de lecture du fichier " << fichierNom << endl;
	
		return false;
	}
	
	// Fini

	fclose(fichier);
	
	return true;
}

/**
* Sauvegarde l'image au format PPM : noir et blanc, 1 bit par pixel
* @param fichierNom : nom du fichier
* @return bool : succs
*/

bool Image::sauvegarderPPM(const string& fichierNom)
{
	unsigned int ecrit;
	
	// Cration fichier

	if (!(fichier = fopen(fichierNom.c_str(), "w")))
	{
		cerr << "Image : Impossible d'ouvrir le fichier " << fichierNom << " en criture" << endl;
		
		return false;
	}

	// criture entete

	fprintf(fichier, "P4\n# CREATOR: M1INFO.\n");
	fprintf(fichier, "%d %d\n", tailleX, tailleY);

	// criture des donnees

	ecrit = fwrite(bitmap, 1, bitmapTaille, fichier);
	
	if(ecrit != bitmapTaille)
	{
		cerr << "Image : Erreur d'criture du fichier " << fichierNom << endl;
	
		return false;
	}

	// Fini

	fclose(fichier);
	
	return true;
}

/**
* Allume un pixel de l'image
* @param x : abscisse
* @param y : ordonne
*/

void Image::pixelAllumer(const unsigned int x, const unsigned int y)
{
	unsigned int pos;
	
	pos = y * tailleX + x;
	bitmap[pos >> 3] &= (1 << (7 - pos%8)) ^ 0xFF;
}

/**
* teint un pixel de l'image
* @param x : abscisse
* @param y : ordonne
*/

void Image::pixelEteindre(const unsigned int x, const unsigned int y)
{
	unsigned int pos;
	
	pos = y * tailleX + x;
	bitmap[pos >> 3] |= 1 << (7 - pos%8);
}

/**
* Renvoie l'tat d'un pixel : allum ou teint
* @param x : abscisse
* @param y : ordonne
* @return bool : tat du pixel
*/

bool Image::getPixel(const unsigned int x, const unsigned int y) const
{
	unsigned int pos;
	
	pos = y * tailleX + x;
	return !(bitmap[pos >> 3] & (1 << (7 - pos%8)));
}

/**
* Renvoie la taille horizontale de l'image
* @return int : taille horizontale de l'image
*/

unsigned int Image::getTailleX() const
{
	return tailleX;
}

/**
* Renvoie la taille verticale de l'image
* @return int : taille verticale de l'image
*/

unsigned int Image::getTailleY() const
{
	return tailleY;
}

/**
* Renvoie un pointeur sur le bitmap
* @return unsigned char * : pointeur sur le bitmap
*/

unsigned char *Image::getBitmap()
{
	return bitmap;
}

/**
* Renvoie la taille des donnes en octets
* @return int : taille des donnes en octets
*/

unsigned int Image::getBitmapTaille() const
{
	return bitmapTaille;
}

/**
* Renvoie une partie d'une image
* @return unsigned char * : pointeur sur le bitmap
*/

Image *Image::partie(const unsigned int ax, const unsigned int ay, const unsigned int bx, const unsigned int by)
{
	Image *imageDest;
	unsigned char *imageDestBitmap;
	int imageDestLargeur, imageDestHauteur;
	int i;
	
	imageDestLargeur = bx - ax + 1;
	imageDestHauteur = by - ay + 1;
	
	if(BPP == 1)
	{
		if(ax % 8 != 0)
		{
			cerr << "Image::partie : ax % 8 != 0" << endl;
			return NULL;
		}
		
		if(bx % 8 != 7)
		{
			cerr << "Image::partie : bx % 8 != 7" << endl;
			return NULL;
		}
		
		imageDest = new Image(imageDestLargeur, imageDestHauteur, BPP);
		imageDestBitmap = imageDest->getBitmap();
		
		for(i=0; i<imageDestHauteur; i++)
			memcpy(imageDestBitmap + (imageDestLargeur >> 3) * i,
				bitmap + (tailleX >> 3) * (ay + i) + (ax >> 3),
				imageDestLargeur >> 3);
	}
	else
		imageDest = NULL;

	return imageDest;
}
