#include "Suivi.h"

using namespace std;

int Suivi::fenetreTailleMax = 20;

/**
* Constructeur d'un objet Suivi
* @param prefixe_i :		prfixe de sauvegarde des images gnres
* @param nbCliche_i : 		nombre de clichs  gnrer
*/

Suivi::Suivi(const string prefixe_i, const unsigned int nbCliche_i) :
		prefixe(string(prefixe_i)), nbCliche(nbCliche_i)
{
	cliche = NULL;
}

/**
* Destructeur d'un objet Suivi
*/

Suivi::~Suivi()
{
	if(cliche)
		delete cliche;
}

/**
* Cre le fichier de suivi des points
* @param fichierNom : nom du fichier
* @return bool : succs
*/

bool Suivi::fichierSuiviCreer(const string& fichierNom)
{
	fichierSuivi.open(fichierNom.c_str(), ios::trunc);
	
	fichierSuivi << nbNoeudX << "," << nbNoeudY << endl;
	fichierSuivi << "#" << endl;
	
	return true;
}

/**
* Enregistre le fichier de suivi des points
* @return bool : succs
*/

bool Suivi::fichierSuiviEnregistrer()
{
	vector<Noeud>::iterator it;
	unsigned int x, y;
	
	it = noeudTab.begin();
	
	while(it != noeudTab.end())
	{
		x = it->getX();
		y = it->getY();
		
		fichierSuivi << x << "," << y;
		
		it++;
		
		if(it != noeudTab.end())
			fichierSuivi << ",";
		else
			fichierSuivi << endl;
	}
	
	return true;
}

/**
* Ferme le fichier de suivi des points
* @return bool : succs
*/

bool Suivi::fichierSuiviFermer()
{
	fichierSuivi.close();
	
	return true;
}

/**
* Charge la position initiale de chaque noeud de la grille
* @param fichierNom : nom du fichier
* @return bool : succs
*/

bool Suivi::chargerPos(const string& fichierNom)
{
	ifstream fichierPos;
	unsigned int i;
	unsigned int x, y;
	
	// Ouverture du fichier en lecture
	
	fichierPos.open(fichierNom.c_str(), ios::in);
	
	if(!fichierPos)
	{
		cerr << "Suivi : Erreur ouverture du fichier " << fichierNom << " en lecture" << endl;
	
		return false;
	}
	
	// Lecture da taille de la grille
	
	fichierPos >> nbNoeudX;
	fichierPos.get();
	fichierPos >> nbNoeudY;
	fichierPos.get();
	
	// Ligne Vide
	
	fichierPos.get();
	fichierPos.get();
	
	// Noeuds
	
	for(i=0; i<nbNoeudX * nbNoeudY; i++)
	{
		fichierPos >> x;
		fichierPos.get();
		fichierPos >> y;
		fichierPos.get();
		
		noeudTab.push_back(Noeud(x, y));
	}
			
	fichierPos.close();
	
	return true;
}

/**
* Suivi d'un noeud
* @param noeud : Noeud  suivre
* @return Bool : succs
*/

bool Suivi::suivreNoeud(Noeud& noeud)
{
	const short incrementX[] = {0, -1, 0, 1}; 	// Incrment selon X
	const short incrementY[] = {-1, 0, 1, 0}; 	// Incrment selon Y
	
	int pos;
	short direction;
	int x, y;
	int i;
	unsigned int bitmapTailleX, bitmapTailleY;

	bitmapTailleX = cliche->getTailleX();
	bitmapTailleY = cliche->getTailleY();
	x = noeud.getX(); 
	y = noeud.getY(); 
	
	noeud.nouvellePosition();

	pos = 0;

	while(true)
	{
		for(i=0; i<=pos/2; i++)
		{
			direction = pos%4;
			
			x += incrementX[direction];
			y += incrementY[direction];
			
			if(x>=0 && x<(int)bitmapTailleX && y>=0 && y<(int)bitmapTailleY && cliche->getPixel(x, y))
			{
				noeud.setX(x);
				noeud.setY(y);
				
				return true;
			}
		}
		
		pos++;
		
		if(pos/2 >= fenetreTailleMax)
			return false;
	}
}

/**
* Suivi des noeuds d'une grille dans une squence d'images
* @return Bool : succs
*/

bool Suivi::suivre()
{
	ostringstream fichierPosNom;
	ostringstream fichierSuiviNom;
	unsigned int i, j;
	
	cliche = new Cliche();
	
	// Chargement de la position initiale de chaque noeud de la grille
	
	fichierPosNom << prefixe << setfill('0') << setw(4) << 0 << ".pos";
	fichierSuiviNom << prefixe << ".suivi";
	
	if(!chargerPos(fichierPosNom.str()))
		return false;
	
	if(!fichierSuiviCreer(fichierSuiviNom.str()))
		return false;
	
	if(!fichierSuiviEnregistrer())
		return false;
		
	// Charge et suit n clichs  partir du disque

	for(i=1; i<nbCliche; i++)
	{
		ostringstream fichierNom;
		
		fichierNom << prefixe << setfill('0') << setw(4) << i << ".ppm";
		
		if(!(cliche->charger(fichierNom.str())))
		{
			cerr << "Suivi : Erreur ouverture fichier " << fichierNom.str() << " en lecture" << endl;
			
			return false;
		}
		
		for(j=0; j<noeudTab.size(); j++)
		{
			if(!suivreNoeud(noeudTab[j]))
			{
				cerr << "Suivi : Erreur de suivi, clich n" << i << ", noeud n" << j << endl;
			
				return false;
			}
		}

		if(!fichierSuiviEnregistrer())
			return false;
	}
	
	if(!fichierSuiviFermer())
		return false;
	
	return true;
}
