#include <assert.h>

#include "SuiviP1Fils.h"

using namespace std;

/**
* Constructeur d'un objet SuiviP1Fils
* @param prefixe_i :		prfixe de sauvegarde des images gnres
* @param nbCliche_i : 		nombre de clichs  gnrer
* @param processusRang_i : 	rang du progessus
* @param processusNb_i : 	nombre de processus
*/

SuiviP1Fils::SuiviP1Fils(const string prefixe_i, const unsigned int nbCliche_i, int processusRang_i, int processusNb_i) :
		SuiviP1(prefixe_i, nbCliche_i, processusRang_i, processusNb_i)
{
}

/**
* Destructeur d'un objet SuiviP1Fils
*/

SuiviP1Fils::~SuiviP1Fils()
{
}

/**
* Destructeur d'un objet SuiviP1Fils
* @return Bool : succs
*/

bool SuiviP1Fils::receptionner()
{
	MPI_Status status;
	int x, y;
	int ax, ay, bx, by;
	int i;
	int res;
	Cliche *c;
	
	res = MPI_Recv(&noeudNum, 1, MPI_INT, 0, 1, MPI_COMM_WORLD, &status);
	assert(res == MPI_SUCCESS);
	
	//cout << "Fils " << processusRang << ", rception de " << noeudNum << " noeuds" << endl;
	
	// Rception de l'ensemble de noeuds  suivre sur ce processus
	
	for(i=0; i<noeudNum; i++)
	{
		res = MPI_Recv(&x, 1, MPI_INT, 0, 2, MPI_COMM_WORLD, &status);
		assert(res == MPI_SUCCESS);
		res = MPI_Recv(&y, 1, MPI_INT, 0, 3, MPI_COMM_WORLD, &status);
		assert(res == MPI_SUCCESS);
		res = MPI_Recv(&ax, 1, MPI_INT, 0, 4, MPI_COMM_WORLD, &status);
		assert(res == MPI_SUCCESS);
		res = MPI_Recv(&ay, 1, MPI_INT, 0, 5, MPI_COMM_WORLD, &status);
		assert(res == MPI_SUCCESS);
		res = MPI_Recv(&bx, 1, MPI_INT, 0, 6, MPI_COMM_WORLD, &status);
		assert(res == MPI_SUCCESS);
		res = MPI_Recv(&by, 1, MPI_INT, 0, 7, MPI_COMM_WORLD, &status);
		assert(res == MPI_SUCCESS);
	
		c = new Cliche(bx - ax + 1, by - ay + 1);
		
		res = MPI_Recv(c->getBitmap(), c->getBitmapTaille(), MPI_CHAR, 0, 8, MPI_COMM_WORLD, &status);
		assert(res == MPI_SUCCESS);
		
		// Sauvegarde en mmoire du clich, du noeud, et des coordonnes de la fentre de calcul
		
		clicheTab.push_back(c);
		noeudTab.push_back(Noeud(x-ax, y-ay));
		aTab.push_back(Point(ax, ay));
		bTab.push_back(Point(bx, by));
	}
	
	// Synchronisation collective
	
	res = MPI_Barrier(MPI_COMM_WORLD);
	assert(res==MPI_SUCCESS);
	
	return true;
}

/**
* Destructeur d'un objet SuiviP1Fils
* @return Bool : succs
*/

bool SuiviP1Fils::calculer()
{
	int i;
	
	for(i=0; i<noeudNum; i++)
	{
		//cout << "Fils " << processusRang << "i-=" <<i<< " "<< noeudTab[i] << "," << aTab[i] << endl;
		
		cliche = clicheTab[i];
		
		if(!suivreNoeud(noeudTab[i]))
		{
			cerr << "calculer : Erreur de suivi, noeud n" << i << endl;
			
			return false;
		}
		
		//cout << "Fils " << processusRang << "i+=" <<i<< " ("<< noeudTab[i].getX() << "," << noeudTab[i].getY() << ")" << endl;
	}

	return true;
}

/**
* Destructeur d'un objet SuiviP1Fils
* @return Bool : succs
*/

bool SuiviP1Fils::envoyer()
{
	int res;
	int x, y;
	int i;
	
	// Envoir du rsultat au processus pre
	
	for(i=0; i<noeudNum; i++)
	{
		x = noeudTab[i].getX() + aTab[i].getX();
		y = noeudTab[i].getY() + aTab[i].getY();
	
		//cout << "Fils " << processusRang << " envoi " << Point(x, y) << endl;
		
		MPI_Send(&x, 1, MPI_INT, 0, 0, MPI_COMM_WORLD);
		MPI_Send(&y, 1, MPI_INT, 0, 1, MPI_COMM_WORLD);
	}
	
	// Synchronisation collective
	
	res = MPI_Barrier(MPI_COMM_WORLD);
	assert(res==MPI_SUCCESS);
	
	clicheTab.clear();
	noeudTab.clear();
	aTab.clear();
	bTab.clear();
	
	return true;
}

/**
* Suivi des noeuds d'une grille dans une squence d'images
* @return Bool : succs
*/

bool SuiviP1Fils::suivre()
{
	unsigned int i;
	
	//cout << "Fils " << processusRang << "nbCliche=" << nbCliche << endl;
	
	for(i=1; i<nbCliche; i++)
	{
		receptionner();
		calculer();
		envoyer();
	}
	
	return true;
}
