#include <assert.h>

#include "SuiviP1Pere.h"

using namespace std;

/**
* Constructeur d'un objet SuiviP1Pere
* @param prefixe_i :		prfixe de sauvegarde des images gnres
* @param nbCliche_i : 		nombre de clichs  gnrer
* @param processusRang_i : 	rang du progessus
* @param processusNb_i : 	nombre de processus
*/

SuiviP1Pere::SuiviP1Pere(const string prefixe_i, const unsigned int nbCliche_i, int processusRang_i, int processusNb_i) :
		SuiviP1(prefixe_i, nbCliche_i, processusRang_i, processusNb_i)
{
}

/**
* Destructeur d'un objet SuiviP1Pere
*/

SuiviP1Pere::~SuiviP1Pere()
{
}

/**
* Partitionnement en ensembles de rgions, un ensemble par processus
* @return bool : succs
*/

bool SuiviP1Pere::partitionner()
{
	unsigned int noeudIndex, noeudIndexMax;
	int noeudNum;
	unsigned int tailleBloc;
	unsigned int i;
	int p;
	Noeud *noeud;
	int x, y;
	int ax, ay, bx, by;
	int res;
	
	// Nombre de noeuds par processeur
	
	tailleBloc = (unsigned int)ceil((double)noeudTab.size() / (processusNb - 1));
	
	//cout << "taille d'un bloc : " << tailleBloc << " rgions" << endl;

	noeudIndex = 0;
	
	for(p=1; p<processusNb; p++)
	{
		// Dtermine l'ensemble des rgions  envoyer 
		
		noeudIndexMax = noeudIndex + tailleBloc;
		
		if(noeudIndexMax > noeudTab.size())
			noeudIndexMax = noeudTab.size();

		noeudNum = noeudIndexMax - noeudIndex;
		
		// Envoie la taille de l'ensemble
		
		MPI_Send(&noeudNum, 1, MPI_INT, p, 1, MPI_COMM_WORLD);
		
		// Envoie chacune des rgions
		
		for(i=noeudIndex; i<noeudIndexMax; i++)
		{
			noeud = &noeudTab[i];
			
			// Calcul des coordonnes de la fentre 
			
			x = noeud->getX();
			y = noeud->getY();
			
			ax = x - fenetreTailleMax / 2;
			ay = y - fenetreTailleMax / 2;
			
			bx = ax + fenetreTailleMax;
			by = ay + fenetreTailleMax;
			
			// Arrondi  8 pixels (on travaille sur des images 1BPP)
			
			ax = ax & 0xFFFFFFF8;
			bx = (bx & 0xFFFFFFF8) + 7;
			
			// Clipping
			
			if(ax < 0)
				ax = 0;
			
			if(ay < 0)
				ay = 0;
			
			if(bx >= (int)cliche->getTailleX())
				ax = cliche->getTailleX() - 1;
			
			if(by >= (int)cliche->getTailleY())
				ay = cliche->getTailleY() - 1;
				
			Image *image = cliche->partie(ax, ay, bx, by);
			
			//cout << "Envoi fils " << p << " noeud " << i << " taille " << image->getBitmapTaille() << endl;
				
			// Envoi de l'image partielle au processus p
			
			//cout <<"pre i="<<i<<" ax="<<ax<<" ay="<<ay<<endl;
			
			MPI_Send(&x, 1, MPI_INT, p, 2, MPI_COMM_WORLD);
			MPI_Send(&y, 1, MPI_INT, p, 3, MPI_COMM_WORLD);
			MPI_Send(&ax, 1, MPI_INT, p, 4, MPI_COMM_WORLD);
			MPI_Send(&ay, 1, MPI_INT, p, 5, MPI_COMM_WORLD);
			MPI_Send(&bx, 1, MPI_INT, p, 6, MPI_COMM_WORLD);
			MPI_Send(&by, 1, MPI_INT, p, 7, MPI_COMM_WORLD);
			MPI_Send(image->getBitmap(), image->getBitmapTaille(), MPI_CHAR, p, 8, MPI_COMM_WORLD);
			
			delete image;
			
			noeudIndex++;
		}
	}
	
	// Synchronisation collective
	
	res = MPI_Barrier(MPI_COMM_WORLD);
	assert(res==MPI_SUCCESS);
	
	return true;
}

/**
* Partitionnement en ensembles de rgions, un ensemble par processus
* @return bool : succs
*/

bool SuiviP1Pere::receptionner()
{
	unsigned int noeudIndex, noeudIndexMax;
	int noeudNum;
	unsigned int tailleBloc;
	unsigned int i;
	int p;
	int res;
	int x, y;
	MPI_Status status;
	
	// Nombre de noeuds par processeur
	
	tailleBloc = (unsigned int)ceil((double)noeudTab.size() / (processusNb - 1));
	
	//cout << "taille d'un bloc : " << tailleBloc << " rgions" << endl;

	noeudIndex = 0;
	
	for(p=1; p<processusNb; p++)
	{
		// Dtermine l'ensemble des rgions  envoyer 
		
		noeudIndexMax = noeudIndex + tailleBloc;
		
		if(noeudIndexMax > noeudTab.size())
			noeudIndexMax = noeudTab.size();

		noeudNum = noeudIndexMax - noeudIndex;
		
		for(i=noeudIndex; i<noeudIndexMax; i++)
		{
			res = MPI_Recv(&x, 1, MPI_INT, p, 0, MPI_COMM_WORLD, &status);
			assert(res == MPI_SUCCESS);
		
			res = MPI_Recv(&y, 1, MPI_INT, p, 1, MPI_COMM_WORLD, &status);
			assert(res == MPI_SUCCESS);
	
			//cout << "Pre " << processusRang << " reu noeud=" << i << " " << Point(x, y) << endl;
			
			noeudTab[i].setX(x);
			noeudTab[i].setY(y);
			
			noeudIndex++;
		}
	}
	
	// Synchronisation collective
	
	res = MPI_Barrier(MPI_COMM_WORLD);
	assert(res==MPI_SUCCESS);
	
	return true;
}

/**
* Suivi des noeuds d'une grille dans une squence d'images
* @return Bool : succs
*/

bool SuiviP1Pere::suivre()
{
	ostringstream fichierPosNom;
	ostringstream fichierSuiviNom;
	unsigned int i;
	
	cliche = new Cliche();
	
	// Chargement de la position initiale de chaque noeud de la grille
	
	fichierPosNom << prefixe << setfill('0') << setw(4) << 0 << ".pos";
	fichierSuiviNom << prefixe << ".suivi";
	
	if(!chargerPos(fichierPosNom.str()))
		return false;
	
	if(!fichierSuiviCreer(fichierSuiviNom.str()))
		return false;
	
	if(!fichierSuiviEnregistrer())
		return false;
		
	// Charge et suit n clichs  partir du disque

	//cout << "Pere nbCliche=" << nbCliche << endl;
	
	for(i=1; i<nbCliche; i++)
	{
		ostringstream fichierNom;
		
		fichierNom << prefixe << setfill('0') << setw(4) << i << ".ppm";
		
		if(!(cliche->charger(fichierNom.str())))
		{
			cerr << "Suivi : Erreur ouverture fichier " << fichierNom.str() << " en lecture" << endl;
			
			return false;
		}
		
		// Partitionnement de l'image en processusNb ensembles de rgions
		
		partitionner();
		receptionner();
		
		/*for(j=0; j<noeudTab.size(); j++)
		{
			if(!suivreNoeud(noeudTab[j]))
			{
				cerr << "Suivi : Erreur de suivi, clich n" << i << ", noeud n" << j << endl;
			
				return false;
			}
		}*/

		if(!fichierSuiviEnregistrer())
			return false;
			
		//cout << "Pre Fin envois image=" << i << endl;
	}
	
	if(!fichierSuiviFermer())
		return false;
	
	return true;
}
